use std::{error::Error, fs, io};

use eyepiece::{Patchable, IPS};

const NAME: &str = env!("CARGO_PKG_NAME");

fn main() -> Result<(), Box<dyn Error>> {
    let args: Vec<_> = std::env::args().skip(1).collect();
    let [ref patch, ref input, ref output] = args[..] else {
        eprintln!("Usage: {NAME} <patch> <in> <out>");
        return Ok(());
    };

    eprintln!("{NAME} v{}", env!("CARGO_PKG_VERSION"));

    let Some(ips) = IPS::parse(&mut fs::File::open(patch)?)? else {
        eprintln!("Magic did not match! Is '{patch}' an IPS file?");
        return Ok(());
    };

    println!("Successfully read patch file '{patch}'");

    let mut outfile = fs::OpenOptions::new()
        .create(true)
        .truncate(false)
        .write(true)
        .open(output)?;

    // FIXME: compare via normalized path equality
    if input != output {
        let mut infile = fs::File::open(input)?;

        let size = io::copy(&mut infile, &mut outfile)?;
        println!("Copied 0x{size:x} bytes from '{input}' to '{output}'");
    }

    outfile.patch(ips)?;

    println!("Patched!");
    Ok(())
}
